import { flow } from 'fp-ts/es6/function';
import { pipe } from 'fp-ts/es6/pipeable';
import * as _ from './internal';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 2.3.0
 */
export var id = _.lensId;
// -------------------------------------------------------------------------------------
// converters
// -------------------------------------------------------------------------------------
/**
 * View a `Lens` as a `Optional`
 *
 * @category converters
 * @since 2.3.0
 */
export var asOptional = _.lensAsOptional;
/**
 * View a `Lens` as a `Traversal`
 *
 * @category converters
 * @since 2.3.0
 */
export var asTraversal = _.lensAsTraversal;
// -------------------------------------------------------------------------------------
// compositions
// -------------------------------------------------------------------------------------
/**
 * Compose a `Lens` with a `Lens`
 *
 * @category compositions
 * @since 2.3.0
 */
export var compose = _.lensComposeLens;
/**
 * Compose a `Lens` with a `Prism`
 *
 * @category compositions
 * @since 2.3.0
 */
export var composePrism = _.lensComposePrism;
/**
 * Compose a `Lens` with an `Optional`
 *
 * @category compositions
 * @since 2.3.0
 */
export var composeOptional = function (ab) { return function (sa) {
    return _.optionalComposeOptional(ab)(asOptional(sa));
}; };
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 2.3.0
 */
export var modify = function (f) { return function (sa) { return function (s) {
    var o = sa.get(s);
    var n = f(o);
    return o === n ? s : sa.set(n)(s);
}; }; };
export function modifyF(F) {
    return function (f) { return function (sa) { return function (s) { return pipe(sa.get(s), f, function (fa) { return F.map(fa, function (a) { return sa.set(a)(s); }); }); }; }; };
}
/**
 * Return a `Optional` from a `Lens` focused on a nullable value
 *
 * @category combinators
 * @since 2.3.0
 */
export var fromNullable = function (sa) {
    return _.lensComposePrism(_.prismFromNullable())(sa);
};
export function filter(predicate) {
    return composePrism(_.prismFromPredicate(predicate));
}
/**
 * Return a `Lens` from a `Lens` and a prop
 *
 * @category combinators
 * @since 2.3.0
 */
export var prop = _.lensProp;
/**
 * Return a `Lens` from a `Lens` and a list of props
 *
 * @category combinators
 * @since 2.3.0
 */
export var props = _.lensProps;
/**
 * Return a `Lens` from a `Lens` and a component
 *
 * @category combinators
 * @since 2.3.0
 */
export var component = _.lensComponent;
/**
 * Return a `Optional` from a `Lens` focused on a `ReadonlyArray`
 *
 * @category combinators
 * @since 2.3.0
 */
export var index = function (i) { return function (sa) {
    return pipe(sa, asOptional, _.optionalComposeOptional(_.indexReadonlyArray().index(i)));
}; };
/**
 * Return a `Optional` from a `Lens` focused on a `ReadonlyRecord` and a key
 *
 * @category combinators
 * @since 2.3.0
 */
export var key = function (key) { return function (sa) {
    return pipe(sa, asOptional, _.optionalComposeOptional(_.indexReadonlyRecord().index(key)));
}; };
/**
 * Return a `Lens` from a `Lens` focused on a `ReadonlyRecord` and a required key
 *
 * @category combinators
 * @since 2.3.0
 */
export var atKey = function (key) { return function (sa) {
    return pipe(sa, compose(_.atReadonlyRecord().at(key)));
}; };
/**
 * Return a `Optional` from a `Lens` focused on the `Some` of a `Option` type
 *
 * @category combinators
 * @since 2.3.0
 */
export var some = 
/*#__PURE__*/
composePrism(_.prismSome());
/**
 * Return a `Optional` from a `Lens` focused on the `Right` of a `Either` type
 *
 * @category combinators
 * @since 2.3.0
 */
export var right = 
/*#__PURE__*/
composePrism(_.prismRight());
/**
 * Return a `Optional` from a `Lens` focused on the `Left` of a `Either` type
 *
 * @category combinators
 * @since 2.3.0
 */
export var left = 
/*#__PURE__*/
composePrism(_.prismLeft());
/**
 * Return a `Traversal` from a `Lens` focused on a `Traversable`
 *
 * @category combinators
 * @since 2.3.0
 */
export function traverse(T) {
    return flow(asTraversal, _.traversalComposeTraversal(_.fromTraversable(T)()));
}
export function findFirst(predicate) {
    return composeOptional(_.findFirst(predicate));
}
// -------------------------------------------------------------------------------------
// pipeables
// -------------------------------------------------------------------------------------
/**
 * @category Invariant
 * @since 2.3.0
 */
export var imap = function (f, g) { return function (ea) {
    return imap_(ea, f, g);
}; };
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
var imap_ = function (ea, ab, ba) { return ({
    get: flow(ea.get, ab),
    set: flow(ba, ea.set)
}); };
/**
 * @category instances
 * @since 2.3.0
 */
export var URI = 'monocle-ts/Lens';
/**
 * @category instances
 * @since 2.3.0
 */
export var invariantLens = {
    URI: URI,
    imap: imap_
};
/**
 * @category instances
 * @since 2.3.0
 */
export var categoryLens = {
    URI: URI,
    compose: function (ab, ea) { return compose(ab)(ea); },
    id: id
};
