"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.atRecord = exports.atReadonlySet = exports.atReadonlyMap = exports.atReadonlyRecord = exports.fromIso = void 0;
var O = require("fp-ts/lib/Option");
var pipeable_1 = require("fp-ts/lib/pipeable");
var RM = require("fp-ts/lib/ReadonlyMap");
var RS = require("fp-ts/lib/ReadonlySet");
var _ = require("./internal");
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * Lift an instance of `At` using an `Iso`
 *
 * @category constructors
 * @since 2.3.0
 */
var fromIso = function (iso) { return function (sia) { return ({
    at: function (i) { return pipeable_1.pipe(iso, _.isoAsLens, _.lensComposeLens(sia.at(i))); }
}); }; };
exports.fromIso = fromIso;
/**
 * @category constructors
 * @since 2.3.7
 */
exports.atReadonlyRecord = _.atReadonlyRecord;
/**
 * @category constructors
 * @since 2.3.7
 */
var atReadonlyMap = function (E) { return function () {
    var lookupE = RM.lookup(E);
    var deleteAtE = RM.deleteAt(E);
    var insertAtE = RM.insertAt(E);
    return {
        at: function (key) { return ({
            get: lookupE(key),
            set: O.fold(function () { return deleteAtE(key); }, function (a) { return insertAtE(key, a); })
        }); }
    };
}; };
exports.atReadonlyMap = atReadonlyMap;
/**
 * @category constructors
 * @since 2.3.7
 */
var atReadonlySet = function (E) {
    var elemE = RS.elem(E);
    var insertE = RS.insert(E);
    var removeE = RS.remove(E);
    return {
        at: function (a) {
            var insert = insertE(a);
            var remove = removeE(a);
            return {
                get: elemE(a),
                set: function (b) { return function (s) { return (b ? insert(s) : remove(s)); }; }
            };
        }
    };
};
exports.atReadonlySet = atReadonlySet;
// -------------------------------------------------------------------------------------
// deprecated
// -------------------------------------------------------------------------------------
/**
 * Use `atReadonlyRecord` instead.
 *
 * @category constructors
 * @since 2.3.2
 * @deprecated
 */
exports.atRecord = _.atReadonlyRecord;
