"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.categoryLens = exports.invariantLens = exports.URI = exports.imap = exports.findFirst = exports.traverse = exports.left = exports.right = exports.some = exports.atKey = exports.key = exports.index = exports.component = exports.props = exports.prop = exports.filter = exports.fromNullable = exports.modifyF = exports.modify = exports.composeOptional = exports.composePrism = exports.compose = exports.asTraversal = exports.asOptional = exports.id = void 0;
var function_1 = require("fp-ts/lib/function");
var pipeable_1 = require("fp-ts/lib/pipeable");
var _ = require("./internal");
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 2.3.0
 */
exports.id = _.lensId;
// -------------------------------------------------------------------------------------
// converters
// -------------------------------------------------------------------------------------
/**
 * View a `Lens` as a `Optional`
 *
 * @category converters
 * @since 2.3.0
 */
exports.asOptional = _.lensAsOptional;
/**
 * View a `Lens` as a `Traversal`
 *
 * @category converters
 * @since 2.3.0
 */
exports.asTraversal = _.lensAsTraversal;
// -------------------------------------------------------------------------------------
// compositions
// -------------------------------------------------------------------------------------
/**
 * Compose a `Lens` with a `Lens`
 *
 * @category compositions
 * @since 2.3.0
 */
exports.compose = _.lensComposeLens;
/**
 * Compose a `Lens` with a `Prism`
 *
 * @category compositions
 * @since 2.3.0
 */
exports.composePrism = _.lensComposePrism;
/**
 * Compose a `Lens` with an `Optional`
 *
 * @category compositions
 * @since 2.3.0
 */
var composeOptional = function (ab) { return function (sa) {
    return _.optionalComposeOptional(ab)(exports.asOptional(sa));
}; };
exports.composeOptional = composeOptional;
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 2.3.0
 */
var modify = function (f) { return function (sa) { return function (s) {
    var o = sa.get(s);
    var n = f(o);
    return o === n ? s : sa.set(n)(s);
}; }; };
exports.modify = modify;
function modifyF(F) {
    return function (f) { return function (sa) { return function (s) { return pipeable_1.pipe(sa.get(s), f, function (fa) { return F.map(fa, function (a) { return sa.set(a)(s); }); }); }; }; };
}
exports.modifyF = modifyF;
/**
 * Return a `Optional` from a `Lens` focused on a nullable value
 *
 * @category combinators
 * @since 2.3.0
 */
var fromNullable = function (sa) {
    return _.lensComposePrism(_.prismFromNullable())(sa);
};
exports.fromNullable = fromNullable;
function filter(predicate) {
    return exports.composePrism(_.prismFromPredicate(predicate));
}
exports.filter = filter;
/**
 * Return a `Lens` from a `Lens` and a prop
 *
 * @category combinators
 * @since 2.3.0
 */
exports.prop = _.lensProp;
/**
 * Return a `Lens` from a `Lens` and a list of props
 *
 * @category combinators
 * @since 2.3.0
 */
exports.props = _.lensProps;
/**
 * Return a `Lens` from a `Lens` and a component
 *
 * @category combinators
 * @since 2.3.0
 */
exports.component = _.lensComponent;
/**
 * Return a `Optional` from a `Lens` focused on a `ReadonlyArray`
 *
 * @category combinators
 * @since 2.3.0
 */
var index = function (i) { return function (sa) {
    return pipeable_1.pipe(sa, exports.asOptional, _.optionalComposeOptional(_.indexReadonlyArray().index(i)));
}; };
exports.index = index;
/**
 * Return a `Optional` from a `Lens` focused on a `ReadonlyRecord` and a key
 *
 * @category combinators
 * @since 2.3.0
 */
var key = function (key) { return function (sa) {
    return pipeable_1.pipe(sa, exports.asOptional, _.optionalComposeOptional(_.indexReadonlyRecord().index(key)));
}; };
exports.key = key;
/**
 * Return a `Lens` from a `Lens` focused on a `ReadonlyRecord` and a required key
 *
 * @category combinators
 * @since 2.3.0
 */
var atKey = function (key) { return function (sa) {
    return pipeable_1.pipe(sa, exports.compose(_.atReadonlyRecord().at(key)));
}; };
exports.atKey = atKey;
/**
 * Return a `Optional` from a `Lens` focused on the `Some` of a `Option` type
 *
 * @category combinators
 * @since 2.3.0
 */
exports.some = 
/*#__PURE__*/
exports.composePrism(_.prismSome());
/**
 * Return a `Optional` from a `Lens` focused on the `Right` of a `Either` type
 *
 * @category combinators
 * @since 2.3.0
 */
exports.right = 
/*#__PURE__*/
exports.composePrism(_.prismRight());
/**
 * Return a `Optional` from a `Lens` focused on the `Left` of a `Either` type
 *
 * @category combinators
 * @since 2.3.0
 */
exports.left = 
/*#__PURE__*/
exports.composePrism(_.prismLeft());
/**
 * Return a `Traversal` from a `Lens` focused on a `Traversable`
 *
 * @category combinators
 * @since 2.3.0
 */
function traverse(T) {
    return function_1.flow(exports.asTraversal, _.traversalComposeTraversal(_.fromTraversable(T)()));
}
exports.traverse = traverse;
function findFirst(predicate) {
    return exports.composeOptional(_.findFirst(predicate));
}
exports.findFirst = findFirst;
// -------------------------------------------------------------------------------------
// pipeables
// -------------------------------------------------------------------------------------
/**
 * @category Invariant
 * @since 2.3.0
 */
var imap = function (f, g) { return function (ea) {
    return imap_(ea, f, g);
}; };
exports.imap = imap;
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
var imap_ = function (ea, ab, ba) { return ({
    get: function_1.flow(ea.get, ab),
    set: function_1.flow(ba, ea.set)
}); };
/**
 * @category instances
 * @since 2.3.0
 */
exports.URI = 'monocle-ts/Lens';
/**
 * @category instances
 * @since 2.3.0
 */
exports.invariantLens = {
    URI: exports.URI,
    imap: imap_
};
/**
 * @category instances
 * @since 2.3.0
 */
exports.categoryLens = {
    URI: exports.URI,
    compose: function (ab, ea) { return exports.compose(ab)(ea); },
    id: exports.id
};
