"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.categoryPrism = exports.invariantPrism = exports.URI = exports.imap = exports.findFirst = exports.traverse = exports.left = exports.right = exports.some = exports.atKey = exports.key = exports.index = exports.component = exports.props = exports.prop = exports.filter = exports.fromNullable = exports.modifyF = exports.modify = exports.modifyOption = exports.set = exports.composeOptional = exports.composeLens = exports.compose = exports.asTraversal = exports.asOptional = exports.fromPredicate = exports.id = void 0;
var function_1 = require("fp-ts/lib/function");
var O = require("fp-ts/lib/Option");
var pipeable_1 = require("fp-ts/lib/pipeable");
var _ = require("./internal");
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 2.3.0
 */
var id = function () { return ({
    getOption: O.some,
    reverseGet: function_1.identity
}); };
exports.id = id;
/**
 * @category constructors
 * @since 2.3.0
 */
exports.fromPredicate = _.prismFromPredicate;
// -------------------------------------------------------------------------------------
// converters
// -------------------------------------------------------------------------------------
/**
 * View a `Prism` as a `Optional`
 *
 * @category converters
 * @since 2.3.0
 */
exports.asOptional = _.prismAsOptional;
/**
 * View a `Prism` as a `Traversal`
 *
 * @category converters
 * @since 2.3.0
 */
exports.asTraversal = _.prismAsTraversal;
// -------------------------------------------------------------------------------------
// compositions
// -------------------------------------------------------------------------------------
/**
 * Compose a `Prism` with a `Prism`
 *
 * @category compositions
 * @since 2.3.0
 */
var compose = function (ab) { return function (sa) { return ({
    getOption: function_1.flow(sa.getOption, O.chain(ab.getOption)),
    reverseGet: function_1.flow(ab.reverseGet, sa.reverseGet)
}); }; };
exports.compose = compose;
/**
 * Compose a `Prism` with a `Lens`
 *
 * @category compositions
 * @since 2.3.0
 */
exports.composeLens = _.prismComposeLens;
/**
 * Compose a `Prism` with an `Optional`
 *
 * @category compositions
 * @since 2.3.0
 */
var composeOptional = function (ab) { return function (sa) {
    return _.optionalComposeOptional(ab)(exports.asOptional(sa));
}; };
exports.composeOptional = composeOptional;
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 2.3.0
 */
exports.set = _.prismSet;
/**
 * @category combinators
 * @since 2.3.0
 */
exports.modifyOption = _.prismModifyOption;
/**
 * @category combinators
 * @since 2.3.0
 */
exports.modify = _.prismModify;
function modifyF(F) {
    return function (f) { return function (sa) { return function (s) {
        return pipeable_1.pipe(sa.getOption(s), O.fold(function () { return F.of(s); }, function (a) { return F.map(f(a), sa.reverseGet); }));
    }; }; };
}
exports.modifyF = modifyF;
/**
 * Return a `Prism` from a `Prism` focused on a nullable value
 *
 * @category combinators
 * @since 2.3.3
 */
exports.fromNullable = 
/*#__PURE__*/
exports.compose(_.prismFromNullable());
function filter(predicate) {
    return exports.compose(_.prismFromPredicate(predicate));
}
exports.filter = filter;
/**
 * Return a `Optional` from a `Prism` and a prop
 *
 * @category combinators
 * @since 2.3.0
 */
var prop = function (prop) {
    return exports.composeLens(pipeable_1.pipe(_.lensId(), _.lensProp(prop)));
};
exports.prop = prop;
/**
 * Return a `Optional` from a `Prism` and a list of props
 *
 * @category combinators
 * @since 2.3.0
 */
var props = function () {
    var props = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        props[_i] = arguments[_i];
    }
    return exports.composeLens(pipeable_1.pipe(_.lensId(), _.lensProps.apply(_, props)));
};
exports.props = props;
/**
 * Return a `Optional` from a `Prism` and a component
 *
 * @category combinators
 * @since 2.3.0
 */
var component = function (prop) { return exports.composeLens(pipeable_1.pipe(_.lensId(), _.lensComponent(prop))); };
exports.component = component;
/**
 * Return a `Optional` from a `Prism` focused on a `ReadonlyArray`
 *
 * @category combinators
 * @since 2.3.0
 */
var index = function (i) { return function (sa) {
    return pipeable_1.pipe(sa, exports.asOptional, _.optionalComposeOptional(_.indexReadonlyArray().index(i)));
}; };
exports.index = index;
/**
 * Return a `Optional` from a `Prism` focused on a `ReadonlyRecord` and a key
 *
 * @category combinators
 * @since 2.3.0
 */
var key = function (key) { return function (sa) {
    return pipeable_1.pipe(sa, exports.asOptional, _.optionalComposeOptional(_.indexReadonlyRecord().index(key)));
}; };
exports.key = key;
/**
 * Return a `Optional` from a `Prism` focused on a `ReadonlyRecord` and a required key
 *
 * @category combinators
 * @since 2.3.0
 */
var atKey = function (key) { return function (sa) {
    return _.prismComposeLens(_.atReadonlyRecord().at(key))(sa);
}; };
exports.atKey = atKey;
/**
 * Return a `Prism` from a `Prism` focused on the `Some` of a `Option` type
 *
 * @category combinators
 * @since 2.3.0
 */
exports.some = 
/*#__PURE__*/
exports.compose(_.prismSome());
/**
 * Return a `Prism` from a `Prism` focused on the `Right` of a `Either` type
 *
 * @category combinators
 * @since 2.3.0
 */
exports.right = 
/*#__PURE__*/
exports.compose(_.prismRight());
/**
 * Return a `Prism` from a `Prism` focused on the `Left` of a `Either` type
 *
 * @category combinators
 * @since 2.3.0
 */
exports.left = 
/*#__PURE__*/
exports.compose(_.prismLeft());
/**
 * Return a `Traversal` from a `Prism` focused on a `Traversable`
 *
 * @category combinators
 * @since 2.3.0
 */
function traverse(T) {
    return function_1.flow(exports.asTraversal, _.traversalComposeTraversal(_.fromTraversable(T)()));
}
exports.traverse = traverse;
function findFirst(predicate) {
    return exports.composeOptional(_.findFirst(predicate));
}
exports.findFirst = findFirst;
// -------------------------------------------------------------------------------------
// pipeables
// -------------------------------------------------------------------------------------
/**
 * @category Invariant
 * @since 2.3.0
 */
var imap = function (f, g) { return function (ea) {
    return imap_(ea, f, g);
}; };
exports.imap = imap;
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
var imap_ = function (ea, ab, ba) { return ({
    getOption: function_1.flow(ea.getOption, O.map(ab)),
    reverseGet: function_1.flow(ba, ea.reverseGet)
}); };
/**
 * @category instances
 * @since 2.3.0
 */
exports.URI = 'monocle-ts/Prism';
/**
 * @category instances
 * @since 2.3.0
 */
exports.invariantPrism = {
    URI: exports.URI,
    imap: imap_
};
/**
 * @category instances
 * @since 2.3.0
 */
exports.categoryPrism = {
    URI: exports.URI,
    compose: function (ab, ea) { return exports.compose(ab)(ea); },
    id: exports.id
};
