"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.categoryTraversal = exports.URI = exports.getAll = exports.fold = exports.foldMap = exports.traverse = exports.left = exports.right = exports.some = exports.atKey = exports.key = exports.index = exports.component = exports.props = exports.prop = exports.filter = exports.set = exports.modify = exports.compose = exports.fromTraversable = exports.id = void 0;
var C = require("fp-ts/lib/Const");
var function_1 = require("fp-ts/lib/function");
var I = require("fp-ts/lib/Identity");
var pipeable_1 = require("fp-ts/lib/pipeable");
var A = require("fp-ts/lib/ReadonlyArray");
var _ = require("./internal");
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 2.3.0
 */
var id = function () { return ({
    modifyF: function (_) { return function (f) { return f; }; }
}); };
exports.id = id;
/**
 * Create a `Traversal` from a `Traversable`
 *
 * @category constructor
 * @since 2.3.0
 */
exports.fromTraversable = _.fromTraversable;
// -------------------------------------------------------------------------------------
// compositions
// -------------------------------------------------------------------------------------
/**
 * Compose a `Traversal` with a `Traversal`
 *
 * @category compositions
 * @since 2.3.0
 */
exports.compose = _.traversalComposeTraversal;
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 2.3.0
 */
var modify = function (f) { return function (sa) {
    return sa.modifyF(I.identity)(f);
}; };
exports.modify = modify;
/**
 * @category combinators
 * @since 2.3.0
 */
var set = function (a) {
    return exports.modify(function () { return a; });
};
exports.set = set;
function filter(predicate) {
    return exports.compose(_.prismAsTraversal(_.prismFromPredicate(predicate)));
}
exports.filter = filter;
/**
 * Return a `Traversal` from a `Traversal` and a prop
 *
 * @category combinators
 * @since 2.3.0
 */
var prop = function (prop) {
    return exports.compose(pipeable_1.pipe(_.lensId(), _.lensProp(prop), _.lensAsTraversal));
};
exports.prop = prop;
/**
 * Return a `Traversal` from a `Traversal` and a list of props
 *
 * @category combinators
 * @since 2.3.0
 */
var props = function () {
    var props = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        props[_i] = arguments[_i];
    }
    return exports.compose(pipeable_1.pipe(_.lensId(), _.lensProps.apply(_, props), _.lensAsTraversal));
};
exports.props = props;
/**
 * Return a `Traversal` from a `Traversal` and a component
 *
 * @category combinators
 * @since 2.3.0
 */
var component = function (prop) {
    return exports.compose(pipeable_1.pipe(_.lensId(), _.lensComponent(prop), _.lensAsTraversal));
};
exports.component = component;
/**
 * Return a `Traversal` from a `Traversal` focused on a `ReadonlyArray`
 *
 * @category combinators
 * @since 2.3.0
 */
var index = function (i) { return function (sa) {
    return pipeable_1.pipe(sa, exports.compose(_.optionalAsTraversal(_.indexReadonlyArray().index(i))));
}; };
exports.index = index;
/**
 * Return a `Traversal` from a `Traversal` focused on a `ReadonlyRecord` and a key
 *
 * @category combinators
 * @since 2.3.0
 */
var key = function (key) { return function (sa) {
    return pipeable_1.pipe(sa, exports.compose(_.optionalAsTraversal(_.indexReadonlyRecord().index(key))));
}; };
exports.key = key;
/**
 * Return a `Traversal` from a `Traversal` focused on a `ReadonlyRecord` and a required key
 *
 * @category combinators
 * @since 2.3.0
 */
var atKey = function (key) { return function (sa) {
    return pipeable_1.pipe(sa, exports.compose(_.lensAsTraversal(_.atReadonlyRecord().at(key))));
}; };
exports.atKey = atKey;
/**
 * Return a `Traversal` from a `Traversal` focused on the `Some` of a `Option` type
 *
 * @category combinators
 * @since 2.3.0
 */
exports.some = 
/*#__PURE__*/
exports.compose(_.prismAsTraversal(_.prismSome()));
/**
 * Return a `Traversal` from a `Traversal` focused on the `Right` of a `Either` type
 *
 * @category combinators
 * @since 2.3.0
 */
exports.right = 
/*#__PURE__*/
exports.compose(_.prismAsTraversal(_.prismRight()));
/**
 * Return a `Traversal` from a `Traversal` focused on the `Left` of a `Either` type
 *
 * @category combinators
 * @since 2.3.0
 */
exports.left = 
/*#__PURE__*/
exports.compose(_.prismAsTraversal(_.prismLeft()));
/**
 * Return a `Traversal` from a `Traversal` focused on a `Traversable`
 *
 * @category combinators
 * @since 2.3.0
 */
function traverse(T) {
    return exports.compose(exports.fromTraversable(T)());
}
exports.traverse = traverse;
/**
 * Map each target to a `Monoid` and combine the results.
 *
 * @category combinators
 * @since 2.3.0
 */
var foldMap = function (M) { return function (f) { return function (sa) {
    return sa.modifyF(C.getApplicative(M))(function (a) { return C.make(f(a)); });
}; }; };
exports.foldMap = foldMap;
/**
 * Map each target to a `Monoid` and combine the results.
 *
 * @category combinators
 * @since 2.3.0
 */
var fold = function (M) { return exports.foldMap(M)(function_1.identity); };
exports.fold = fold;
/**
 * Get all the targets of a `Traversal`.
 *
 * @category combinators
 * @since 2.3.0
 */
var getAll = function (s) { return function (sa) {
    return exports.foldMap(A.getMonoid())(function (a) { return [a]; })(sa)(s);
}; };
exports.getAll = getAll;
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 2.3.0
 */
exports.URI = 'monocle-ts/Traversal';
/**
 * @category instances
 * @since 2.3.0
 */
exports.categoryTraversal = {
    URI: exports.URI,
    compose: function (ab, ea) { return exports.compose(ab)(ea); },
    id: exports.id
};
